#!/usr/bin/env python
# -*- coding: iso-8859-1 -*-

"An OpenID login provider."

# Site map imports.

from WebStack.Generic import ContentType
from WebStack.Resources.OpenIDLogin import Authenticator
from WebStack.Resources.ResourceMap import MapResource
from WebStack.Resources.Selectors import EncodingSelector, PathSelector
from WebStack.Resources.Static import StringResource
import XSLForms.Resources.OpenIDLogin
from XSLForms.Resources.WebResources import output, prepare_resources as pr, resources
import urllib

# Configuration settings.

encoding = "utf-8"

# Resource classes.

class LoginResource(XSLForms.Resources.OpenIDLogin.OpenIDLoginResource):

    "Special login screen for this application."

    resource_dir = resources(__file__)
    template_resources = {
        "login" : output("login_template.xhtml"),
        "success" : output("success_template.xhtml")
        }

# Site map initialisation.

def get_site(host, use_redirect=1):

    "Return a simple Web site resource."

    # NOTE: Support "special" username due to stupid HTML 4 restrictions.

    special_username = urllib.quote(u"".encode("utf-8"))

    # Authentication objects.

    credentials = [
        # Local identifier, username, password
        ((host +  "/badger",  "badger"), "abc"),
        ((host +  "/vole",    "vole"),   "xyz"),
        ((host + u"/",    u""),   u""),
        ((host +  "/badger2", "badger"), "abc"),
        ((host +  "/vole2",   "vole"),   "xyz"),
        ((host + u"/2",   u""),   u""),
        ((host +  "/" + special_username, u""), u""),
        ]
    login_authenticator = Authenticator(credentials)

    # Get a login resource.

    login_resource = LoginResource(host, login_authenticator, use_redirect=use_redirect)

    # A container for the login resource and user pages.

    resource = MapResource({
        "login" : login_resource,

        # Some local OpenID pages.

        "badger" :
            StringResource("""
                <html>
                  <head>
                    <meta http-equiv="Content-Type" content="text/html; charset="iso-8859-1" />
                    <link rel="openid2.provider openid.server" href="%s/login" />
                    <link rel="openid2.local_id openid.delegate" href="%s/badger" />
                    <title>Badger's Home Page</title>
                  </head>
                  <body>
                    <p>Home page for the OpenID authenticated user, <code>badger</code>.</p>
                  </body>
                </html>
            """ % (host, host), ContentType("text/html", "iso-8859-1")),
        u"" :
            StringResource(u"""
                <html>
                  <head>
                    <meta http-equiv="Content-Type" content="text/html; charset="iso-8859-1" />
                    <link rel="openid2.provider openid.server" href="%s/login" />
                    <link rel="openid2.local_id openid.delegate" href="%s/%s" />
                    <title>Home Page for </title>
                  </head>
                  <body>
                    <p>Home page for the OpenID authenticated user, <code></code>.</p>
                  </body>
                </html>
            """ % (host, host, special_username), ContentType("text/html", "iso-8859-1")),
        "vole" :
            StringResource("""
                <html>
                  <head>
                    <meta http-equiv="Content-Type" content="text/html; charset="iso-8859-1" />
                    <link rel="openid2.provider openid.server" href="%s/login" />
                    <link rel="openid2.local_id openid.delegate" href="%s/vole" />
                    <title>Vole's Home Page</title>
                  </head>
                  <body>
                    <p>Home page for the OpenID authenticated user, <code>vole</code>.</p>
                  </body>
                </html>
            """ % (host, host), ContentType("text/html", "iso-8859-1")),

        # OpenID 2.0 resources.
        # See: http://docs.oasis-open.org/xri/2.0/specs/cd02/xri-resolution-V2.0-cd-02.html#_Ref129424065
        #      ("3.3 Media Types for XRI Resolution")
        # See: http://openid.net/specs/openid-authentication-2_0.html#discovery
        #      ("7.3. Discovery")

        "badger2" :
            StringResource("""
                <?xml version="1.0"?>
                <Service xmlns="xri://$xrd*($v*2.0)">
                  <Type>http://specs.openid.net/auth/2.0/signon</Type>
                  <URI>%s/login</URI>
                  <LocalID>%s/badger2</LocalID>
                </Service>
            """ % (host, host), ContentType("application/xrd+xml", "iso-8859-1")),
        u"2" :
            StringResource(u"""
                <?xml version="1.0" encoding="iso-8859-1"?>
                <Service xmlns="xri://$xrd*($v*2.0)">
                  <Type>http://specs.openid.net/auth/2.0/signon</Type>
                  <URI>%s/login</URI>
                  <LocalID>%s/2</LocalID>
                </Service>
            """ % (host, host), ContentType("application/xrd+xml", "iso-8859-1")),
        "vole2" :
            StringResource("""
                <?xml version="1.0"?>
                <Service xmlns="xri://$xrd*($v*2.0)">
                  <Type>http://specs.openid.net/auth/2.0/signon</Type>
                  <URI>%s/login</URI>
                  <LocalID>%s/vole2</LocalID>
                </Service>
            """ % (host, host), ContentType("application/xrd+xml", "iso-8859-1")),

        })

    # Wrap the resource up with information about the application root.

    return EncodingSelector(PathSelector(resource), encoding)

# Resource preparation ahead of time - useful for making installations.

def prepare_resources():
    for cls in [LoginResource]:
        pr(cls)

# vim: tabstop=4 expandtab shiftwidth=4
